@@ -1,16 +1,34 @@
+import testbase
 from testbase import PersistTest
 import unittest, sys, os, time
 
-from pysqlite2 import dbapi2 as sqlite
 import sqlalchemy.pool as pool
-import sqlalchemy.exceptions as exceptions 
+import sqlalchemy.exceptions as exceptions
+
+class MockDBAPI(object):
+    def __init__(self):
+        self.throw_error = False
+    def connect(self, argument):
+        if self.throw_error:
+            raise Exception("couldnt connect !")
+        return MockConnection()
+class MockConnection(object):
+    def close(self):
+        pass
+    def cursor(self):
+        return MockCursor()
+class MockCursor(object):
+    def close(self):
+        pass
+mock_dbapi = MockDBAPI()
+         
 class PoolTest(PersistTest):
     
     def setUp(self):
         pool.clear_managers()
-        
+
     def testmanager(self):
-        manager = pool.manage(sqlite)
+        manager = pool.manage(mock_dbapi)
         
         connection = manager.connect('foo.db')
         connection2 = manager.connect('foo.db')
@@ -22,7 +40,7 @@ class PoolTest(PersistTest):
         self.assert_(connection2 is not connection3)
 
     def testbadargs(self):
-        manager = pool.manage(sqlite)
+        manager = pool.manage(mock_dbapi)
 
         try:
             connection = manager.connect(None)
@@ -30,7 +48,7 @@ class PoolTest(PersistTest):
             pass
     
     def testnonthreadlocalmanager(self):
-        manager = pool.manage(sqlite, use_threadlocal = False)
+        manager = pool.manage(mock_dbapi, use_threadlocal = False)
         
         connection = manager.connect('foo.db')
         connection2 = manager.connect('foo.db')
@@ -47,8 +65,7 @@ class PoolTest(PersistTest):
         self._do_testqueuepool(useclose=True)
 
     def _do_testqueuepool(self, useclose=False):
-
-        p = pool.QueuePool(creator = lambda: sqlite.connect('foo.db'), pool_size = 3, max_overflow = -1, use_threadlocal = False, echo = False)
+        p = pool.QueuePool(creator = lambda: mock_dbapi.connect('foo.db'), pool_size = 3, max_overflow = -1, use_threadlocal = False, echo = False)
     
         def status(pool):
             tup = (pool.size(), pool.checkedin(), pool.overflow(), pool.checkedout())
@@ -91,7 +108,7 @@ class PoolTest(PersistTest):
         self.assert_(status(p) == (3, 2, 0, 1))
     
     def test_timeout(self):
-        p = pool.QueuePool(creator = lambda: sqlite.connect('foo.db'), pool_size = 3, max_overflow = 0, use_threadlocal = False, echo = False, timeout=2)
+        p = pool.QueuePool(creator = lambda: mock_dbapi.connect('foo.db'), pool_size = 3, max_overflow = 0, use_threadlocal = False, echo = False, timeout=2)
         c1 = p.get()
         c2 = p.get()
         c3 = p.get()
@@ -102,6 +119,77 @@ class PoolTest(PersistTest):
         except exceptions.TimeoutError, e:
             assert int(time.time() - now) == 2
         
+    def test_mixed_close(self):
+        p = pool.QueuePool(creator = lambda: mock_dbapi.connect('foo.db'), pool_size = 3, max_overflow = -1, use_threadlocal = True, echo=True)
+        c1 = p.connect()
+        c2 = p.connect()
+        assert c1 is c2
+        c1.close()
+        c2 = None
+        assert p.checkedout() == 1
+        c1 = None
+        assert p.checkedout() == 0
+    
+    def test_trick_the_counter(self):
+        """this is a "flaw" in the connection pool; since threadlocal uses a single ConnectionFairy per thread
+        with an open/close counter, you can fool the counter into giving you a ConnectionFairy with an
+        ambiguous counter.  i.e. its not true reference counting."""
+        p = pool.QueuePool(creator = lambda: mock_dbapi.connect('foo.db'), pool_size = 3, max_overflow = -1, use_threadlocal = True, echo=True)
+        c1 = p.connect()
+        c2 = p.connect()
+        assert c1 is c2
+        c1.close()
+        c2 = p.connect()
+        c2.close()
+        self.assert_(p.checkedout() != 0)
+
+        c2.close()
+        self.assert_(p.checkedout() == 0)
+
+    def test_recycle(self):
+        p = pool.QueuePool(creator = lambda: mock_dbapi.connect('foo.db'), pool_size = 1, max_overflow = 0, use_threadlocal = False, echo=True, recycle=3)
+        
+        c1 = p.connect()
+        c_id = id(c1.connection)
+        c1.close()
+        c2 = p.connect()
+        assert id(c2.connection) == c_id
+        c2.close()
+        time.sleep(3)
+        c3= p.connect()
+        assert id(c3.connection) != c_id
+    
+    def test_invalidate(self):
+        dbapi = MockDBAPI()
+        p = pool.QueuePool(creator = lambda: dbapi.connect('foo.db'), pool_size = 1, max_overflow = 0, use_threadlocal = False, echo=True)
+        c1 = p.connect()
+        c_id = id(c1.connection)
+        c1.close(); c1=None
+
+        c1 = p.connect()
+        assert id(c1.connection) == c_id
+        c1.invalidate()
+        c1 = None
+        
+        c1 = p.connect()
+        assert id(c1.connection) != c_id
+
+    def test_reconnect(self):
+        dbapi = MockDBAPI()
+        p = pool.QueuePool(creator = lambda: dbapi.connect('foo.db'), pool_size = 1, max_overflow = 0, use_threadlocal = False, echo=True)
+        c1 = p.connect()
+        c_id = id(c1.connection)
+        c1.close(); c1=None
+
+        c1 = p.connect()
+        assert id(c1.connection) == c_id
+        dbapi.raise_error = True
+        c1.invalidate()
+        c1 = None
+
+        c1 = p.connect()
+        assert id(c1.connection) != c_id
+        
     def testthreadlocal_del(self):
         self._do_testthreadlocal(useclose=False)
 
@@ -110,8 +198,8 @@ class PoolTest(PersistTest):
 
     def _do_testthreadlocal(self, useclose=False):
         for p in (
-            pool.QueuePool(creator = lambda: sqlite.connect('foo.db'), pool_size = 3, max_overflow = -1, use_threadlocal = True, echo = False),
-            pool.SingletonThreadPool(creator = lambda: sqlite.connect('foo.db'), use_threadlocal = True)
+            pool.QueuePool(creator = lambda: mock_dbapi.connect('foo.db'), pool_size = 3, max_overflow = -1, use_threadlocal = True, echo = True),
+            pool.SingletonThreadPool(creator = lambda: mock_dbapi.connect('foo.db'), use_threadlocal = True)
         ):   
             c1 = p.connect()
             c2 = p.connect()
@@ -130,8 +218,6 @@ class PoolTest(PersistTest):
             else:
                 c2 = None
         
-            c3 = None
-            
             if useclose:
                 c1 = p.connect()
                 c2 = p.connect()
@@ -140,8 +226,8 @@ class PoolTest(PersistTest):
                 c2.close()
                 self.assert_(c1.connection is not None)
                 c1.close()
-            else:
-                c1 = c2 = c3 = None
+
+            c1 = c2 = c3 = None
             
             # extra tests with QueuePool to insure connections get __del__()ed when dereferenced
             if isinstance(p, pool.QueuePool):
@@ -158,9 +244,6 @@ class PoolTest(PersistTest):
             
     def tearDown(self):
        pool.clear_managers()
-       for file in ('foo.db', 'bar.db'):
-            if os.access(file, os.F_OK):
-                os.remove(file)
         
         
 if __name__ == "__main__":
